@@ -1,14 +1,13 @@
 import argparse
+import json
 import logging
 import os
 import sys
 from datetime import datetime
-from importlib.machinery import SourceFileLoader
 
 from . import config
 from .classes.nodes import ChannelNode
 from .commands import authenticate_user
-from .commands import uploadchannel
 from .commands import uploadchannel_wrapper
 from .exceptions import InvalidUsageException
 from .exceptions import raise_for_invalid_channel
@@ -27,64 +26,71 @@ from .utils.metadata_provider import DEFAULT_CONTENT_INFO_FILENAME
 from .utils.metadata_provider import DEFAULT_EXERCISE_QUESTIONS_INFO_FILENAME
 from .utils.metadata_provider import DEFAULT_EXERCISES_INFO_FILENAME
 from .utils.tokens import get_content_curation_token
-# for JsonTreeChef chef
-# for LineCook chef
 
 
-# SUSHI CHEF BASE CLASS (and backward compatibiliry)
+
+
+
+# SUSHI CHEF BASE CLASS
 ################################################################################
 
-class BaseChef(object):
+class SushiChef(object):
     """
-    The base class that parses command line arguments for sushichef scripts.
-    Sushi chef sctipts call the `main` method as the entry point, which in turn
-    calls the `run` method to performs all the work (see `uploadchannel`).
-
-    This class also provides backward compaibility with old chef scripts.
-    When invoking the a sushi chef script using the old API: \
-        python -m ricecooker uploadchannel chef_script.py --token=123 ...
-    an instance of this class with `compatibility_mode = True` will be created.
-    Calling `BaseChef.run` will call the function `construct_channel` in `chef_module`.
+    This is the base class that all content integration scripts should subclass.
+    Sushi chef scripts call the `main` method as the entry point, which in turn
+    calls the `run` method to do the work (see `uploadchannel` in `commands.py`).
     """
+    CHEF_RUN_DATA = config.CHEF_DATA_DEFAULT  # loaded from chefdata/chef_data.json
+    TREES_DATA_DIR = config.TREES_DATA_DIR    # tree archives and JsonTreeChef inputs
 
-    def __init__(self, *args, compatibility_mode=False, **kwargs):
-        """
-        Setup argparse arguments.
-        """
-        self.compatibility_mode = compatibility_mode
 
-        # argparse setup
+    def __init__(self, *args, **kwargs):
+        """
+        The SushiChef initialization concerns maintly parsing command line args.
+        Overrride this method in your sushi chef class to add custom arguments.
+        """
+
+        # ARGPARSE SETUP
+        # We don't want to add argparse help if subclass has an __init__ method
+        subclasses = self.__class__.__mro__[:-2]     # all subclasses after this
+        if any(['__init__' in c.__dict__.keys() for c in subclasses]):
+            add_parser_help = False    # assume subclass' __init__ will add help
+        else:
+            add_parser_help = True
         parser = argparse.ArgumentParser(
-            description="Ricecooker puts your content in the conten server.",
-            add_help=(self.__class__ == BaseChef)  # only add help if not subclassed
+            description="Chef script for uploading content to Kolibri Studio.",
+            add_help=add_parser_help,
         )
-        parser.add_argument('command', nargs='?', default='uploadchannel', help='Desired action: dryrun or uploadchannel')
-        if self.compatibility_mode:
-            parser.add_argument('chef_script', help='Path to chef script file')
-        parser.add_argument('--token', default='#',                   help='Authorization token (can be token or path to file with token)')
-        parser.add_argument('-u', '--update', action='store_true',    help='Force re-download of files (skip .ricecookerfilecache/ check)')
-        parser.add_argument('-v', '--verbose', action='store_true', default=True, help='Verbose mode')
-        parser.add_argument('--quiet', action='store_true',           help='Print only errors to stderr')
-        parser.add_argument('--warn', action='store_true',            help='Print warnings to stderr')
-        parser.add_argument('--debug', action='store_true',           help='Print debugging log info to stderr')
-        parser.add_argument('--compress', action='store_true',        help='Compress high resolution videos to low resolution videos')
-        parser.add_argument('--thumbnails', action='store_true',      help='Automatically generate thumbnails for topics')
-        parser.add_argument('--download-attempts',type=int,default=3, help='Maximum number of times to retry downloading files')
-        rrgroup = parser.add_mutually_exclusive_group()
-        rrgroup.add_argument('--reset', action='store_true',          help='Restart session, overwriting previous session (cannot be used with --resume flag)')
-        rrgroup.add_argument('--resume', action='store_true',         help='Resume from ricecooker step (cannot be used with --reset flag)')
+        self.arg_parser = parser  # save as class attr. for subclasses to extend
+        # ARGS
+        parser.add_argument('command', nargs='?', default='uploadchannel', help='Desired action: dryrun or uploadchannel (default).')
+        parser.add_argument('--token', default='#',                   help='Studio API Access Token (specify wither the token value or the path of a file that contains the token).')
+        parser.add_argument('-u', '--update', action='store_true',    help='Force file re-download (skip .ricecookerfilecache/).')
+        parser.add_argument('--debug', action='store_true',           help='Print extra debugging infomation.')
+        parser.add_argument('-v', '--verbose', action='store_true', default=True, help='Verbose mode (default).')
+        parser.add_argument('--warn', action='store_true',            help='Print errors and warnings.')
+        parser.add_argument('--quiet', action='store_true',           help='Print only errors.')
+        parser.add_argument('--compress', action='store_true',        help='Compress videos using ffmpeg -crf=32 -b:a 32k mono.')
+        parser.add_argument('--thumbnails', action='store_true',      help='Automatically generate thumbnails for content nodes.')
+        parser.add_argument('--download-attempts',type=int,default=3, help='Maximum number of times to retry downloading files.')
+        parser.add_argument('--resume', action='store_true',          help='Resume chef session from a specified step.')
         allsteps = [step.name.upper() for step in Status]
-        parser.add_argument('--step',choices=allsteps,default='LAST', help='Step to resume progress from (must be used with --resume flag)')
-        parser.add_argument('--prompt', action='store_true',          help='Prompt user to open the channel after creating it')
-        parser.add_argument('--stage', dest='stage_deprecated', action='store_true',
-                            help='(Deprecated.) Stage updated content for review. This flag is now the default and has been kept solely to maintain compatibility. Use --deploy to immediately push changes.')
+        parser.add_argument('--step',choices=allsteps,default='LAST', help='Step to resume progress from (use with the --resume).')
+        parser.add_argument('--prompt', action='store_true',          help='Prompt user to open the channel after the chef run.')
         parser.add_argument('--deploy', dest='stage', action='store_false',
-                            help='Immediately deploy changes to channel\'s main tree. This operation will delete the previous channel content once upload completes. Default (recommended) behavior is to post new tree for review.')
-        parser.add_argument('--publish', action='store_true',         help='Publish newly uploaded version of the channel')
-        parser.add_argument('--sample', type=int, metavar='SIZE',     help='Upload a sample of SIZE content nodes from the channel')
-
+                                                                      help='Immediately deploy changes to channel\'s main tree. This operation will overwrite the previous channel content. Use only during development.')
+        parser.add_argument('--publish', action='store_true',         help='Publish newly uploaded version of the channel.')
+        parser.add_argument('--sample', type=int, metavar='SIZE',     help='Upload a sample of SIZE nodes from the channel.')
+        parser.add_argument('--reset', dest="reset_deprecated", action='store_true',
+                                                                      help='(deprecated) Restarting the chef run is the default.')
+        parser.add_argument('--stage', dest='stage_deprecated', action='store_true',
+                                                                      help='(deprecated) Stage updated content for review. Uploading a staging tree is now the default behavior. Use --deploy to upload to the main tree.')
+        parser.add_argument('--daemon', action='store_true', help='Run chef in daemon mode lisenting to commands.')
+        parser.add_argument('--nomonitor', action='store_true', help='Disable SushiBar progress monitoring.')
+        parser.add_argument('--cmdsock', help='Local command socket (for cronjobs).')
         # [OPTIONS] --- extra key=value options are supported, but do not appear in help
-        self.arg_parser = parser
+
+        self.load_chef_data()
 
 
     def parse_args_and_options(self):
@@ -113,10 +119,12 @@ class BaseChef(object):
             args['command'] = 'uploadchannel'
             options_list.append(command_arg)  # put command_arg where it belongs
 
-
         # Print CLI deprecation warnings info
         if args['stage_deprecated']:
-            config.LOGGER.warning('DEPRECATION WARNING: --stage is now default, so the --stage flag has been deprecated and will be removed in ricecooker 1.0.')
+            config.LOGGER.warning('DEPRECATION WARNING: --stage is now the default bevavior. The --stage flag has been deprecated and will be removed in ricecooker 1.0.')
+        if args['reset_deprecated']:
+            config.LOGGER.warning(
+                'DEPRECATION WARNING: --reset is now the default bevavior. The --reset flag has been deprecated and will be removed in ricecooker 1.0.')
         if args['publish'] and args['stage']:
             raise InvalidUsageException('The --publish argument must be used together with --deploy argument.')
         logging_args = [key for key in ['quiet', 'warn', 'debug'] if args[key]]
@@ -135,8 +143,6 @@ class BaseChef(object):
 
         if args['command'] == 'dryrun':
             args['nomonitor'] = True    # no Sushibar logs and progress tracking
-            if not args['resume']:
-                args['reset'] = True    # set the --reset flag to avoid prompt
 
         # Parse additional keyword arguments from `options_list`
         options = {}
@@ -148,16 +154,9 @@ class BaseChef(object):
                 msg = "Invalid option '{0}': use [key]=[value] format (no whitespace)".format(preoption)
                 raise InvalidUsageException(msg)
 
-        # For compatibility mode, we check the chef script file exists and load it
-        if self.compatibility_mode:
-            try:
-                # Try to load the chef_script as a module
-                self.chef_module = SourceFileLoader("mod", args['chef_script']).load_module()
-            except FileNotFoundError as e:
-                raise InvalidUsageException('Error: must specify `chef_module` for compatibility_mode')
-
         return args, options
 
+
     def config_logger(self, args, options):
         """
         Set up stream (stderr), local file logging (logs/yyyy-mm-dd__HHMM.log),
@@ -201,65 +200,17 @@ class BaseChef(object):
             config.LOGGER.warning('Unable to use remote logging due to: %s' % e)
 
 
-    def pre_run(self, args, options):
-        """
-        This function is called before the Chef's `run` mehod is called.
-        By default this function does nothing, but subclass can use this hook to
-        run prerequisite tasks.
-        Args:
-            args (dict): chef command line arguments
-            options (dict): extra key=value options given on command line
-        """
-        pass
-
-
-    def run(self, args, options):
-        """
-        This function calls uploadchannel which performs all the run steps:
-          - Create ChannelNode
-          - Pupulate Tree with TopicNodes, ContentNodes, and associated File objects
-          - .
-          - ..
-          - ...
-
-        Args:
-            args (dict): ricecooker command line arguments
-            options (dict): extra key=value options given on command line
-        """
-        self.pre_run(args, options)
-        args_and_options = args.copy()
-        args_and_options.update(options)
-        uploadchannel(self, **args_and_options)
-
-
     def get_channel(self, **kwargs):
         """
-        Call chef script's get_channel method in compatibility mode
-        ...or...
-        Create a `ChannelNode` from the Chef's `channel_info` class attribute.
-
+        This method creates an empty `ChannelNode` object based on info from the
+        chef class' `channel_info` attribute. A subclass can ovveride this method
+        in cases where channel metadata is dynamic and depends on `kwargs`.
         Args:
-            kwargs (dict): additional keyword arguments that `uploadchannel` received
-        Returns: channel created from get_channel method or None
+            kwargs (dict): additional keyword arguments given to `uploadchannel`
+        Returns: an empty `ChannelNode` that contains all the channel metadata
         """
-        if self.compatibility_mode:
-            # For pre-sushibar scritps that do not implement `get_channel`,
-            # we must check it this function exists before calling it...
-            if hasattr(self.chef_module, 'get_channel'):
-                config.LOGGER.info("Calling get_channel... ")
-                # Create channel (using the function in the chef script)
-                channel = self.chef_module.get_channel(**kwargs)
-            # For chefs with a `create_channel` method instead of `get_channel`
-            if hasattr(self.chef_module, 'create_channel'):
-                config.LOGGER.info("Calling create_channel... ")
-                # Create channel (using the function in the chef script)
-                channel = self.chef_module.create_channel(**kwargs)
-            else:
-                channel = None  # since no channel info, SushiBar functionality will be disabled...
-            return channel
-
-        elif hasattr(self, 'channel_info'):
-            # Before going any further, make sure there aren't template id values in channel_info
+        if hasattr(self, 'channel_info'):
+            # Make sure we're not using the template id values in `channel_info`
             template_domains = ['<yourdomain.org>']
             using_template_domain = self.channel_info['CHANNEL_SOURCE_DOMAIN'] in template_domains
             if using_template_domain:
@@ -273,84 +224,30 @@ class BaseChef(object):
             if using_template_domain or using_template_source_id:
                sys.exit(1)
 
-            # If a sublass has an `channel_info` attribute (a dict) it doesn't need
+            # If a sublass has an `channel_info` attribute (dict) it doesn't need
             # to define a `get_channel` method and instead rely on this code:
             channel = ChannelNode(
                 source_domain=self.channel_info['CHANNEL_SOURCE_DOMAIN'],
                 source_id=self.channel_info['CHANNEL_SOURCE_ID'],
                 title=self.channel_info['CHANNEL_TITLE'],
+                tagline=self.channel_info.get('CHANNEL_TAGLINE'),
                 thumbnail=self.channel_info.get('CHANNEL_THUMBNAIL'),
                 language=self.channel_info.get('CHANNEL_LANGUAGE'),
                 description=self.channel_info.get('CHANNEL_DESCRIPTION'),
             )
             return channel
-
         else:
-            raise NotImplementedError('BaseChef must overrride the get_channel method')
+            raise NotImplementedError('Subclass must define get_channel method or have a channel_info (dict) attribute.')
 
 
     def construct_channel(self, **kwargs):
         """
-        Calls chef script's construct_channel method. Used only in compatibility mode.
+        This should be overriden by the chef script's construct_channel method.
         Args:
-            kwargs (dict): additional keyword arguments that `uploadchannel` received
-        Returns: channel populated from construct_channel method
+            kwargs (dict): additional keyword arguments given to `uploadchannel`
+        Returns: a `ChannelNode` object representing the populated topic tree
         """
-        if self.compatibility_mode:
-            # Constuct channel (using function from imported chef script)
-            config.LOGGER.info("Populating channel... ")
-            channel = self.chef_module.construct_channel(**kwargs)
-            return channel
-        else:
-            raise NotImplementedError('Your chef class must overrride the construct_channel method')
-
-
-    def main(self):
-        args, options = self.parse_args_and_options()
-        self.config_logger(args, options)
-        self.run(args, options)
-
-
-
-
-
-# THE DEFAULT SUSHI CHEF
-################################################################################
-
-class SushiChef(BaseChef):
-    """
-    This is the main class that all suchi chefs should subclass. This class uses
-    remote logging, remote stage and progress reporting, and remote commands.
-    The `SushiChef` class a subclass of the `BaseChef` and supports the same
-    command line arguments, additionally handles arguments for Sushi Bar server.
-    Sushi chef scripts call the `main` method as the entry point, which in turn
-    calls the `run` method to performs all the work (see `uploadchannel`).
-    """
-
-    def __init__(self, *args, **kwargs):
-        """
-        The SushiChef supports all the command line args of a BaseChef and more.
-        Overrride this method in your sushi chef class to add custom arguments.
-        """
-        super(SushiChef, self).__init__(*args, **kwargs)
-
-        # We don't want to add argparse help if subclass has an __init__ method
-        subclasses = self.__class__.__mro__[:-3]     # all subclasses after this
-        if any(['__init__' in c.__dict__.keys() for c in subclasses]):
-            add_parser_help = False    # assume subclass' __init__ will add help
-        else:
-            add_parser_help = True
-
-        self.arg_parser = argparse.ArgumentParser(
-            description="Chef scripts upload content to the Kolibri Studio server.",
-            add_help=add_parser_help,
-            parents=[self.arg_parser]
-        )
-        self.arg_parser.add_argument('--daemon', action='store_true', help='Run chef in daemon mode')
-        self.arg_parser.add_argument('--nomonitor', action='store_true', help='Disable SushiBar progress monitoring')
-        self.arg_parser.add_argument('--cmdsock', help='Local command socket (for cronjobs)')
-        # self.arg_parser.add_argument('--sushibar', help='Hostname of SushiBar server (e.g. "sushibar.learningequality.org")')
-        # TODO: --bartoken
+        raise NotImplementedError('Chef subclass must implement this method')
 
 
     def daemon_mode(self, args, options):
@@ -369,6 +266,36 @@ class SushiChef(BaseChef):
         cws.join()
 
 
+    def load_chef_data(self):
+        if os.path.exists(config.DATA_PATH):
+            self.CHEF_RUN_DATA = json.load(open(config.DATA_PATH))
+
+
+    def save_channel_tree_as_json(self, channel):
+        filename = os.path.join(self.TREES_DATA_DIR, '{}.json'.format(self.CHEF_RUN_DATA['current_run']))
+        os.makedirs(self.TREES_DATA_DIR, exist_ok=True)
+        json.dump(channel.get_json_tree(), open(filename, 'w'), indent=2)
+        self.CHEF_RUN_DATA['tree_archives']['previous'] = self.CHEF_RUN_DATA['tree_archives']['current']
+        self.CHEF_RUN_DATA['tree_archives']['current'] = filename.replace(os.getcwd() + '/', '')
+        self.save_chef_data()
+
+
+    def save_chef_data(self):
+        json.dump(self.CHEF_RUN_DATA, open(config.DATA_PATH, 'w'), indent=2)
+
+
+    def pre_run(self, args, options):
+        """
+        This function is called before the Chef's `run` mehod is called.
+        By default this function does nothing, but subclass can use this hook to
+        run prerequisite tasks.
+        Args:
+            args (dict): chef command line arguments
+            options (dict): extra key=value options given on command line
+        """
+        pass
+
+
     def run(self, args, options):
         """
         This function calls uploadchannel which performs all the run steps:
@@ -379,11 +306,20 @@ class SushiChef(BaseChef):
         args_copy = args.copy()
         args_copy['token'] = args_copy['token'][0:6] + '...'
         config.LOGGER.info('In SushiChef.run method. args=' + str(args_copy) + ' options=' + str(options))
+
+        run_id = datetime.now().strftime("%Y-%m-%d__%H%M")
+        self.CHEF_RUN_DATA['current_run'] = run_id
+        self.CHEF_RUN_DATA['runs'].append({'id': run_id})
+
+        # TODO(Kevin): move self.download_content() call here
         self.pre_run(args, options)
         uploadchannel_wrapper(self, args, options)
 
 
     def main(self):
+        """
+        Main entry point that content integration scripts should call.
+        """
         args, options = self.parse_args_and_options()
         self.config_logger(args, options)
         if args['daemon']:
@@ -392,6 +328,8 @@ class SushiChef(BaseChef):
             self.run(args, options)
 
 
+
+
 # JSON TREE CHEF
 ################################################################################
 
@@ -435,8 +373,6 @@ class JsonTreeChef(SushiChef):
     Each object in the json tree correponds to a TopicNode, a ContentNode that
     contains a Files or an Exercise that contains Question.
     """
-    DATA_DIR = 'chefdata'
-    TREES_DATA_DIR = os.path.join(DATA_DIR, 'trees')
     RICECOOKER_JSON_TREE = 'ricecooker_json_tree.json'
 
     def pre_run(self, args, options):
@@ -567,7 +503,3 @@ class LineCook(JsonTreeChef):
         kwargs.update(options)
         json_tree_path = self.get_json_tree_path(**kwargs)
         build_ricecooker_json_tree(args, options, self.metadata_provider, json_tree_path)
-
-    # UNCOMMENT BELOW TO DISABLE CHANNEL UPLOAD
-    # def run(self, args, options):
-    #     self.pre_run(args, options)
