@@ -203,6 +203,15 @@ class Node(object):
         for child in self.children:
             child.print_tree(indent + 1)
 
+    def get_json_tree(self):
+        tree = self.to_dict()
+        if len(self.children) > 0:
+            tree['children'] = []
+            for child in self.children:
+                tree['children'].append(child.get_json_tree())
+
+        return tree
+
     def validate_tree(self):
         """
         Validate all nodes in this tree recusively.
@@ -250,10 +259,11 @@ class ChannelNode(Node):
             files ([<File>]): list of file objects for node (optional)
     """
     kind = "Channel"
-    def __init__(self, source_id, source_domain, *args, **kwargs):
+    def __init__(self, source_id, source_domain, tagline=None, *args, **kwargs):
         # Map parameters to model variables
         self.source_domain = source_domain
         self.source_id = source_id
+        self.tagline = tagline
 
         super(ChannelNode, self).__init__(*args, **kwargs)
 
@@ -270,6 +280,9 @@ class ChannelNode(Node):
         if self.description and len(self.description) > config.MAX_DESCRIPTION_LENGTH:
             config.print_truncate("description", self.source_id, self.description, kind=self.kind)
             self.description = self.description[:config.MAX_DESCRIPTION_LENGTH]
+        if self.tagline and len(self.tagline) > config.MAX_TAGLINE_LENGTH:
+            config.print_truncate("tagline", self.source_id, self.tagline, kind=self.kind)
+            self.tagline = self.tagline[:config.MAX_TAGLINE_LENGTH]
         super(ChannelNode, self).truncate_fields()
 
     def to_dict(self):
@@ -283,6 +296,7 @@ class ChannelNode(Node):
             "thumbnail": self.thumbnail.filename if self.thumbnail else None,
             "language" : self.language,
             "description": self.description or "",
+            "tagline": self.tagline or "",
             "license": self.license,
             "source_domain": self.source_domain,
             "source_id": self.source_id,
@@ -393,7 +407,7 @@ class TreeNode(Node):
             "license_description": None,
             "copyright_holder": "",
             "questions": [],
-            "extra_fields": {},
+            "extra_fields": json.dumps(self.extra_fields),
         }
 
     def validate(self):
@@ -588,7 +602,8 @@ class VideoNode(ContentNode):
                         new_files.append(file)
                         language_codes_seen.add(language_code)
                     else:
-                        config.LOGGER.warning('Skipping duplicate subs for ' + language_code + ' from path ' + file.path)
+                        file_info = file.path if hasattr(file, 'path') else file.youtube_url
+                        config.LOGGER.warning('Skipping duplicate subs for ' + language_code + ' from ' + file_info)
                 else:
                     new_files.append(file)
             self.files = new_files
