@@ -18,7 +18,6 @@ except NameError:
     pass
 
 
-
 def uploadchannel_wrapper(chef, args, options):
     """
     Call `uploadchannel` with SushiBar monitoring and progress reporting enabled.
@@ -42,17 +41,16 @@ def uploadchannel_wrapper(chef, args, options):
             config.SUSHI_BAR_CLIENT.close()
 
 
-def uploadchannel(chef, command='uploadchannel', update=False, thumbnails=False, download_attempts=3, resume=False, reset=False, step=Status.LAST.name, token="#", prompt=False, publish=False, compress=False, stage=False, **kwargs):
-    """ uploadchannel: Upload channel to Kolibri Studio server
+def uploadchannel(chef, command='uploadchannel', update=False, thumbnails=False, download_attempts=3, resume=False, step=Status.LAST.name, token="#", prompt=False, publish=False, compress=False, stage=False, **kwargs):
+    """ uploadchannel: Upload channel to Kolibri Studio
         Args:
-            chef (BaseChef or subclass): class that implements the construct_channel method
+            chef (SushiChef subclass): class that implements the construct_channel method
             command (str): the action we want to perform in this run
             update (bool): indicates whether to re-download files (optional)
             thumbnails (bool): indicates whether to automatically derive thumbnails from content (optional)
             download_attempts (int): number of times to retry downloading files (optional)
             resume (bool): indicates whether to resume last session automatically (optional)
             step (str): step to resume process from (optional)
-            reset (bool): indicates whether to start session from beginning automatically (optional)
             token (str): content server authorization token
             prompt (bool): indicates whether to prompt user to open channel when done (optional)
             publish (bool): indicates whether to automatically publish channel (optional)
@@ -89,7 +87,7 @@ def uploadchannel(chef, command='uploadchannel', update=False, thumbnails=False,
 
     # Set up progress tracker
     config.PROGRESS_MANAGER = RestoreManager()
-    if (reset or not config.PROGRESS_MANAGER.check_for_session()) and step.upper() != Status.DONE.name:
+    if (not resume or not config.PROGRESS_MANAGER.check_for_session()) and step.upper() != Status.DONE.name:
         config.PROGRESS_MANAGER.init_session()
     else:
         if resume or prompt_yes_or_no('Previous session detected. Would you like to resume your last session?'):
@@ -99,6 +97,9 @@ def uploadchannel(chef, command='uploadchannel', update=False, thumbnails=False,
         else:
             config.PROGRESS_MANAGER.init_session()
 
+    if hasattr(chef, 'download_content'):
+        chef.download_content()
+
     # Construct channel if it hasn't been constructed already
     if config.PROGRESS_MANAGER.get_status_val() <= Status.CONSTRUCT_CHANNEL.value:
         config.LOGGER.info("Calling construct_channel... ")
@@ -119,6 +120,9 @@ def uploadchannel(chef, command='uploadchannel', update=False, thumbnails=False,
         config.LOGGER.info("Downloading files...")
         config.PROGRESS_MANAGER.set_files(*process_tree_files(tree))
 
+    # Save the data about the current run in chefdata/
+    chef.save_channel_tree_as_json(channel)
+
     if command == 'dryrun':
         config.LOGGER.info('Command is dryrun so we are not uploading chanel.')
         return
