@@ -1,10 +1,17 @@
-# Settings for rice cooker
+"""
+Settings and global config values for ricecooker.
+"""
+import atexit
 import hashlib
 import logging.config
 import os
-
 import requests
 from requests_file import FileAdapter
+import shutil
+import socket
+import tempfile
+
+
 
 UPDATE = False
 COMPRESS = False
@@ -14,9 +21,14 @@ PROGRESS_MANAGER = None
 SUSHI_BAR_CLIENT = None
 STAGE = False
 
-# Don't use this - call logging.getLogger(__name__) from each
-# individual module. Logging is configured centrally by calling
-# setup_logging()
+# Sometimes chef runs will get stuck indefinitely waiting on data from SSL conn,
+# so we add a timeout value as suggested in https://stackoverflow.com/a/30771995
+socket.setdefaulttimeout(20)
+
+
+# Logging is configured globally by calling setup_logging() in the chef's `main`
+# Use this as `from ricecooker.config import LOGGER` in your suchichef.py code,
+# or use the stanard `logging.getLogger(__name__)` to get a namespaced logger.
 LOGGER = logging.getLogger()
 
 
@@ -30,7 +42,7 @@ def setup_logging(level=logging.INFO, main_log=None, error_log=None, add_loggers
     Set up logging, useful to call from your sushi chef main script
 
     :param level: Minimum default level for all loggers and handlers
-    :param main_log: Main log (typically added in chefs.BaseChef)
+    :param main_log: Main log (typically added in chefs.SushiChef)
     :param error_log: Name of file to log (append) errors in
     :param add_loggers: An iterable of other loggers to configure (['scrapy'])
     """
@@ -114,15 +126,17 @@ def setup_logging(level=logging.INFO, main_log=None, error_log=None, add_loggers
     logging.getLogger("PIL.PngImagePlugin").setLevel(logging.WARNING)
 
 
-# Setup default logging - can be called again to reconfigure
+# Setup default logging. This is required so we have a basic logging setup until
+# prope user-confgured logging is configured in `SushiChef.config_logger`.
 setup_logging()
 
 
 # Domain and file store location for uploading to production Studio server
+DEFAULT_DOMAIN = "https://api.studio.learningequality.org"
 DOMAIN_ENV = os.getenv('STUDIO_URL', None)
 if DOMAIN_ENV is None:  # check old ENV varable for backward compatibility
     DOMAIN_ENV = os.getenv('CONTENTWORKSHOP_URL', None)
-DOMAIN = DOMAIN_ENV if DOMAIN_ENV else "https://api.studio.learningequality.org"
+DOMAIN = DOMAIN_ENV if DOMAIN_ENV else DEFAULT_DOMAIN
 if DOMAIN.endswith('/'):
     DOMAIN = DOMAIN.rstrip('/')
 FILE_STORE_LOCATION = hashlib.md5(DOMAIN.encode('utf-8')).hexdigest()
@@ -199,12 +213,50 @@ SUSHI_BAR_LOGS_URL = "{domain}/logs/{run_id}/"
 SUSHI_BAR_CONTROL_URL = "{domain}/control/{channel_id}/"
 
 
+# Automatic temporary direcotry cleanup
+chef_temp_dir = os.path.join(os.getcwd(), '.ricecooker-temp')
+
+@atexit.register
+def delete_temp_dir():
+    if os.path.exists(chef_temp_dir):
+        LOGGER.info("Deleting chef temp files at {}".format(chef_temp_dir))
+        shutil.rmtree(chef_temp_dir)
+
+# While in most cases a chef run will clean up after itself, make sure that if it didn't,
+# temp files from the old run are deleted so that they do not accumulate.
+delete_temp_dir()
+
+# If tempdir is set already, that means the user has explicitly chosen a location for temp storage
+if not tempfile.tempdir:
+    os.makedirs(chef_temp_dir)
+    LOGGER.info("Setting chef temp dir to {}".format(chef_temp_dir))
+    # Store all chef temp files in one dir to avoid issues with temp or even primary storage filling up
+    # because of failure by the chef to clean up temp files manually.
+    tempfile.tempdir = chef_temp_dir
+
+
+# Record data about past chef runs in chefdata/ dir
+DATA_DIR = 'chefdata'
+DATA_FILENAME = 'chef_data.json'
+DATA_PATH = os.path.join(DATA_DIR, DATA_FILENAME)
+CHEF_DATA_DEFAULT = {
+    'current_run': None,
+    'runs': [],
+    'tree_archives': {
+        'previous': None,
+        'current': None
+    }
+}
+TREES_DATA_DIR = os.path.join(DATA_DIR, 'trees')
+
+
 # Character limits based on Kolibri models
 TRUNCATE_MSG = "\t\t{kind} {id}: {field} {value} is too long - max {max} characters (truncating)"
 
 MAX_TITLE_LENGTH = 200
 MAX_SOURCE_ID_LENGTH = 200
 MAX_DESCRIPTION_LENGTH = 400
+MAX_TAGLINE_LENGTH = 150
 MAX_AUTHOR_LENGTH = 200
 MAX_AGGREGATOR_LENGTH = 200
 MAX_PROVIDER_LENGTH = 200
@@ -229,6 +281,11 @@ MAX_CHAR_LIMITS = {
         "field": "description",
         "max": MAX_DESCRIPTION_LENGTH
     },
+    "tagline": {
+        "kind": "Channel",
+        "field": "tagline",
+        "max": MAX_TAGLINE_LENGTH
+    },
     "author": {
         "kind": "Node",
         "field": "author",
