@@ -56,7 +56,8 @@ def channel_data(channel_node_id, channel_content_id, domain_namespace, channel_
         "id": channel_node_id.hex,
         "name": "Channel",
         "thumbnail": None,
-        "description": "Channel description",
+        "description": "Long channel description that can be up to 400 characters.",
+        "tagline": "Short channel description to show on content cards",
         "license": None,
         "source_domain": domain_namespace,
         "source_id": channel_source_id,
@@ -71,6 +72,7 @@ def channel(domain_namespace, channel_source_id, channel_data):
         domain_namespace,
         title=channel_data['name'],
         description=channel_data['description'],
+        tagline=channel_data['tagline'],
         language=channel_data['language']
     )
     return channel
@@ -94,7 +96,9 @@ def invalid_channel(channel_source_id, domain_namespace):
 @pytest.fixture
 def base_data(channel_domain_namespace, title):
     """
-    The dictionary returned by this function resembles outpout of `to_dict` method.
+    The dictionary returned by this function resembles output of `to_dict` except
+    for `extra_fields` which is dict when used as input kwarg but serialized as
+    string when sending to Studio.
     """
     return {
         "kind": None,
@@ -105,7 +109,7 @@ def base_data(channel_domain_namespace, title):
         "files" : [],
         "tags": [],
         "questions": [],
-        "extra_fields": "{}",  # because Ricecookr uses `json.dumps` for this field
+        "extra_fields": {},  # dict as input kwarg, but json.dumps-ed in to_dict
         "license": None,
         "copyright_holder": "",
         "license_description": None,
@@ -174,8 +178,7 @@ def topic_data(base_data, channel_domain_namespace, channel_node_id):
     topic_data = copy.deepcopy(base_data)
     ids_dict = genrate_random_ids(channel_domain_namespace, channel_node_id)
     topic_data.update(ids_dict)
-    topic_data.update({ "kind": content_kinds.TOPIC,
-                        "extra_fields": {} })
+    topic_data.update({ "kind": content_kinds.TOPIC })
     return topic_data
 
 @pytest.fixture
